/**
 * Der Bankkontokorrent kann ueberzogen werden, d.h. der Saldo kann kleiner als 0 sein
 * (balance < 0). Ein negativer Banksaldo ergibt eine Zinsbelastung. Der
 * Sollzins bei einem negativen Saldo ist wesentlich hoeher als der Habenzins bei
 * einem positivem Saldo.
 */
class CurrentAccount extends Account {

	/**
	 * Habenzinssatz
	 */
	protected static float POSITIVE_INTEREST_RATE = 0.025f;

	/**
	 * Sollzinssatz
	 */
	protected static float NEGATIVE_INTEREST_RATE = 9;

	/**
	 * Konstruktor, initialisiert das Konto.
	 */
	public CurrentAccount(int accountNo) {
		super(accountNo);
	}

	/**
	 * Hebt Geld vom Konto ab und benachrichtigt allfaellige Beobachter von der
	 * Aenderung des Saldos. 
	 * PRE: true 
	 * POST: balance == balance@PRE - amount
	 */
	public void withdraw(float amount) {
		float oldBalance = balance;
		balance -= amount;
		
		// Beobachter benachrichtigen
		this.setChanged();
		notifyObservers(new Float(oldBalance));
	}

	/**
	 * Berechnet den aufgelaufenen Zins seit der letzten Abrechnung, ohne
	 * Beruecksichtiung der Kontenbewegungen. 
	 * PRE: true 
	 * POST: if (balance < 0)
	 *        then result == NEGATIVE_INTEREST_RATE * balance 
	 *        else result == POSITIVE_INTEREST_RATE * balance
	 */
	public float calculateInterests() {
		float retVal = ((balance < 0) ? NEGATIVE_INTEREST_RATE
				: POSITIVE_INTEREST_RATE)
				* balance / 100.0f;
		return retVal;
	}
}