/**
 * Ein Sparkonto muss immer einen positiven Saldo aufweisen 
 * (balance >= 0). Ein Sparkonto hat dafuer einen groesseren Habenzins 
 * als ein Bankkontokorrent.
 * 
 * INV: balance >= 0
 */
class SavingsAccount extends Account {
	/**
	 * Habenzinssatz
	 */
	protected static float INTEREST_RATE = 1.25f;

	/**
	 * Konstruktor 
	 * PRE: true 
	 * POST: true
	 */
	public SavingsAccount(int accountNo) {
		super(accountNo);
	}

	/**
	 * Berechnet den Habenzins seit der letzten Zinsverbuchung. 
	 * PRE: true 
	 * POST: result == balance * INTEREST_RATE / 100.0f
	 */
	public float calculateInterests() {
		return balance * INTEREST_RATE / 100.0f;
	}

	/**
	 * Hebt Geld vom Konto ab. Dabei darf der Saldo nicht kleiner werden als 0.
	 * Ist der abzuhebende Betrag zu gross, so
	 * wird eine RuntimeException geworfen. 
	 * PRE: true 
	 * POST: if (balance - amount < 0) then
	 *         exception RuntimeException 
	 *       else 
	 *         balance == balance@PRE - amount
	 * OBLIGATION: Fangen einer allfaelligen
	 *       java.lang.RuntimeException, welche bei einem einem Sparkontosaldo beim 
	 *       Unterschreiten von 0 ausgloest wird.
	 */
	public void withdraw(float amount) {
		float oldBalance = balance;
		if (balance - amount < 0) {
			throw new RuntimeException("Balance of " + balance
					+ " to low for withdrawing amount of " + amount);
		} else {
			balance -= amount;
		}
		// Beobachter benachrichtigen
		setChanged();
		notifyObservers(new Float(oldBalance));
	}
}